@@ -169,6 +169,13 @@ class EventPacketIn(event.EventBase):
         self.msg = msg
 
 
+# For Python3 compatibility
+# Note: The following is the official workaround for cmp() in Python2.
+# https://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
+def cmp(a, b):
+    return (a > b) - (a < b)
+
+
 class Stp(app_manager.RyuApp):
     """ STP(spanning tree) library. """
 
@@ -290,6 +297,11 @@ class Stp(app_manager.RyuApp):
                 bridge.port_delete(port.port_no)
             else:
                 assert reason is dp.ofproto.OFPPR_MODIFY
+                if bridge.dp.ports[port.port_no].state == port.state:
+                    # Do nothing
+                    self.logger.debug('[port=%d] Link status not changed.',
+                                      port.port_no, extra=dpid_str)
+                    return
                 if link_down_flg:
                     self.logger.info('[port=%d] Link down.',
                                      port.port_no, extra=dpid_str)
@@ -351,7 +363,7 @@ class Stp(app_manager.RyuApp):
 
     @staticmethod
     def _cmp_value(value1, value2):
-        result = cmp(value1, value2)
+        result = cmp(str(value1), str(value2))
         if result < 0:
             return SUPERIOR
         elif result == 0:
