@@ -472,7 +472,8 @@ class CoreService(Factory, Activity):
         if (host, port) in self.bmpclients:
             bmpclient = self.bmpclients[(host, port)]
             if bmpclient.started:
-                LOG.warn("bmpclient is already running for %s:%s", host, port)
+                LOG.warning("bmpclient is already running for %s:%s",
+                            host, port)
                 return False
         bmpclient = BMPClient(self, host, port)
         self.bmpclients[(host, port)] = bmpclient
@@ -481,7 +482,7 @@ class CoreService(Factory, Activity):
 
     def stop_bmp(self, host, port):
         if (host, port) not in self.bmpclients:
-            LOG.warn("no bmpclient is running for %s:%s", host, port)
+            LOG.warning("no bmpclient is running for %s:%s", host, port)
             return False
 
         bmpclient = self.bmpclients[(host, port)]
